@@ -30,6 +30,7 @@ static int aes_key_blob_has_valid_mkvp(struct zpc_aes_key *aes_key,
 static int aes_key_blob_is_pkey_extractable(struct zpc_aes_key *aes_key,
 								const unsigned char *buf, size_t buflen);
 static int aes_key_add_ep11_header(struct zpc_aes_key *aes_key);
+static int aes_key_blob_has_a_session(struct zpc_aes_key *aes_key);
 
 int
 zpc_aes_key_alloc(struct zpc_aes_key **aes_key)
@@ -418,7 +419,7 @@ int
 zpc_aes_key_import_clear(struct zpc_aes_key *aes_key, const unsigned char *key)
 {
 	struct pkey_clr2seck2 clr2seck2;
-	unsigned int flags;
+	unsigned int flags, hdr_to_add = 0;
 	int rc, rv;
 
 	UNUSED(rv);
@@ -469,31 +470,62 @@ zpc_aes_key_import_clear(struct zpc_aes_key *aes_key, const unsigned char *key)
 	clr2seck2.apqns = aes_key->apqns;
 	clr2seck2.apqn_entries = aes_key->napqns;
 	clr2seck2.type = aes_key->type;
+	if (aes_key->type == ZPC_AES_KEY_TYPE_EP11)
+		clr2seck2.type = TOKVER_EP11_AES_WITH_HEADER; /* 0x06 */
 	clr2seck2.size = aes_key->keysize;
 	clr2seck2.keygenflags = flags;
 	memcpy(&clr2seck2.clrkey, key, aes_key->keysize / 8);
 	clr2seck2.key = aes_key->cur.sec;
 	clr2seck2.keylen = sizeof(aes_key->cur.sec);
 
+	/*
+	 * If it's an EP11 key, we first try to import the clear key as a type 6 key
+	 * (TOKVER_EP11_AES_WITH_HEADER). This may fail on older kernels. If it
+	 * fails, we try to import as a type 3 (TOKVER_EP11_AES) key and add the
+	 * type 6 header manually below. If successful, we end up with a type 6
+	 * key in both cases.
+	 */
 	rc = ioctl(pkeyfd, PKEY_CLR2SECK2, &clr2seck2);
+	DEBUG("ioctl PKEY_CLR2SECK2 as aes ep11 type 6 key returned %d", rc);
 	if (rc != 0) {
-		rc = ZPC_ERROR_IOCTLCLR2SECK2;
-		goto ret;
+		if (aes_key->type == ZPC_AES_KEY_TYPE_EP11) {
+			DEBUG("ioctl PKEY_CLR2SECK2 as aes ep11 type 6 key returned %d", rc);
+			/* Retry with a type 3 key */
+			clr2seck2.type = TOKVER_EP11_AES; /* 0x03 */
+			rc = ioctl(pkeyfd, PKEY_CLR2SECK2, &clr2seck2);
+			DEBUG("ioctl PKEY_CLR2SECK2 as aes ep11 type 3 key returned %d", rc);
+			if (rc != 0) {
+				rc = ZPC_ERROR_IOCTLCLR2SECK2;
+				goto ret;
+			}
+			hdr_to_add = 1;
+		} else {
+			rc = ZPC_ERROR_IOCTLCLR2SECK2;
+			goto ret;
+		}
 	}
 
 	aes_key->cur.seclen = clr2seck2.keylen;
 
-	rc = aes_key_sec2prot(aes_key, AES_KEY_SEC_CUR);
+	rc = aes_key_clr2prot(aes_key, key, aes_key->keysize / 8);
 	if (rc) {
-		goto ret;
+		rc = aes_key_sec2prot(aes_key, AES_KEY_SEC_CUR);
+		if (rc) {
+			goto ret;
+		}
 	}
 
-	if (aes_key->type == ZPC_AES_KEY_TYPE_EP11) {
+	if (aes_key->type == ZPC_AES_KEY_TYPE_EP11 && hdr_to_add) {
 		rc = aes_key_add_ep11_header(aes_key);
 		if (rc)
 			goto ret;
 	}
 
+	if (aes_key->type == ZPC_AES_KEY_TYPE_EP11 && aes_key_blob_has_a_session(aes_key))
+		DEBUG("This aes ep11 key has a session id");
+	else
+		DEBUG("This aes ep11 key has no session id");
+
 	DEBUG("aes key at %p: key set", aes_key);
 	aes_key->key_set = 1;
 	rc = 0;
@@ -656,7 +688,7 @@ zpc_aes_key_generate(struct zpc_aes_key *aes_key)
 {
 	struct pkey_genseck2 genseck2;
 	struct pkey_genprotk genprotk;
-	unsigned int flags;
+	unsigned int flags, hdr_to_add = 0;
 	int rc, rv;
 
 	UNUSED(rv);
@@ -733,15 +765,38 @@ zpc_aes_key_generate(struct zpc_aes_key *aes_key)
 	genseck2.apqns = aes_key->apqns;
 	genseck2.apqn_entries = aes_key->napqns;
 	genseck2.type = aes_key->type;
+	if (aes_key->type == ZPC_AES_KEY_TYPE_EP11)
+		genseck2.type = TOKVER_EP11_AES_WITH_HEADER; /* 0x06 */
 	genseck2.size = aes_key->keysize;
 	genseck2.keygenflags = flags;
 	genseck2.key = aes_key->cur.sec;
 	genseck2.keylen = sizeof(aes_key->cur.sec);
 
+	/*
+	 * If it's an EP11 key, we first try to generate a type 6 key
+	 * (TOKVER_EP11_AES_WITH_HEADER). This may fail on older kernels. If it
+	 * fails, we try to generate a type 3 (TOKVER_EP11_AES) key and add the
+	 * type 6 header manually below. If successful, we end up with a type 6
+	 * key in both cases.
+	 */
 	rc = ioctl(pkeyfd, PKEY_GENSECK2, &genseck2);
+	DEBUG("ioctl PKEY_GENSECK2 as aes ep11 type 6 key returned %d", rc);
 	if (rc != 0) {
-		rc = ZPC_ERROR_IOCTLGENSECK2;
-		goto ret;
+		if (aes_key->type == ZPC_AES_KEY_TYPE_EP11) {
+			DEBUG("ioctl PKEY_GENSECK2 as aes ep11 type 6 key returned %d", rc);
+			/* Retry to generate a type 3 key */
+			genseck2.type = TOKVER_EP11_AES; /* 0x03 */;
+			rc = ioctl(pkeyfd, PKEY_GENSECK2, &genseck2);
+			DEBUG("ioctl PKEY_GENSECK2 as aes ep11 type 3 key returned %d", rc);
+			if (rc != 0) {
+				rc = ZPC_ERROR_IOCTLGENSECK2;
+				goto ret;
+			}
+			hdr_to_add = 1;
+		} else {
+			rc = ZPC_ERROR_IOCTLGENSECK2;
+			goto ret;
+		}
 	}
 
 	aes_key->cur.seclen = genseck2.keylen;
@@ -750,12 +805,17 @@ zpc_aes_key_generate(struct zpc_aes_key *aes_key)
 	if (rc)
 		goto ret;
 
-	if (aes_key->type == ZPC_AES_KEY_TYPE_EP11) {
+	if (aes_key->type == ZPC_AES_KEY_TYPE_EP11 && hdr_to_add) {
 		rc = aes_key_add_ep11_header(aes_key);
 		if (rc)
 			goto ret;
 	}
 
+	if (aes_key->type == ZPC_AES_KEY_TYPE_EP11 && aes_key_blob_has_a_session(aes_key))
+		DEBUG("This aes ep11 key has a session id");
+	else
+		DEBUG("This aes ep11 key has no session id");
+
 	DEBUG("aes key at %p: key set to generated secure key", aes_key);
 	aes_key->key_set = 1;
 	rc = 0;
@@ -1101,6 +1161,47 @@ int aes_key_sec2prot(struct zpc_aes_key *aes_key, enum aes_key_sec sec)
 	return aes_key_sec2prot_without_header(aes_key, sec);
 }
 
+int aes_key_clr2prot(struct zpc_aes_key *aes_key, const unsigned char *key,
+					unsigned int keylen)
+{
+	struct pkey_kblob2pkey3 io;
+	unsigned char buf[sizeof(struct clearkeytoken) + 32];
+	struct clearkeytoken *clrtok = (struct clearkeytoken *)&buf;
+	int rc;
+
+	memset(buf, 0, sizeof(buf));
+	clrtok->version = 0x02;
+	switch (keylen) {
+	case 16:
+		clrtok->keytype = PKEY_KEYTYPE_AES_128;
+		break;
+	case 24:
+		clrtok->keytype = PKEY_KEYTYPE_AES_192;
+		break;
+	default:
+		clrtok->keytype = PKEY_KEYTYPE_AES_256;
+		break;
+	}
+	memcpy(clrtok->clearkey, key, keylen);
+	clrtok->len = keylen;
+
+	memset(&io, 0, sizeof(io));
+	io.key = buf;
+	io.keylen = sizeof(struct clearkeytoken) + keylen;
+	io.apqns = aes_key->apqns;
+	io.apqn_entries = aes_key->napqns;
+	io.pkeytype = aes_key->type;
+	io.pkeylen = sizeof(aes_key->prot.protkey);
+	io.pkey = (unsigned char *)&aes_key->prot.protkey;
+
+	rc = ioctl(pkeyfd, PKEY_KBLOB2PROTK3, &io);
+	if (rc != 0) {
+		return ZPC_ERROR_IOCTLBLOB2PROTK3;
+	}
+
+	return 0;
+}
+
 int
 aes_key_check(const struct zpc_aes_key *aes_key)
 {
@@ -1198,7 +1299,18 @@ static int aes_key_add_ep11_header(struct zpc_aes_key *aes_key)
 	ep11hdr = (struct ep11kblob_header *)aes_key->cur.sec;
 	ep11hdr->len = sizeof(struct ep11kblob_header) + aes_key->cur.seclen;
 	ep11hdr->version = TOKVER_EP11_AES_WITH_HEADER;
+	ep11hdr->bitlen = aes_key->keysize;
 	aes_key->cur.seclen = ep11hdr->len;
 
 	return 0;
 }
+
+static int aes_key_blob_has_a_session(struct zpc_aes_key *aes_key)
+{
+	u8 session[32] = { 0, };
+
+	if (memcmp(aes_key->cur.sec + sizeof(struct ep11kblob_header), session, 32) == 0)
+		return 0;
+
+	return 1;
+}
